#############################################################################
  
  # BayClone2 :  A Bayesian Sequence and Copy Number Caller for Subclones Using NGS Data
  # Copyright (C) <2014>  <Juhee Lee>
  # This file is part of BayClone2.

  #  BayClone2 is free software: you can redistribute it and/or modify
  #  it under the terms of the GNU General Public License as published by
  #  the Free Software Foundation, either version 3 of the License, or
  #  (at your option) any later version.

  #  BayClone2 is distributed in the hope that it will be useful,
  #  but WITHOUT ANY WARRANTY; without even the implied warranty of
  #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  #  GNU General Public License for more details.

  #  You should have received a copy of the GNU General Public License
  #  along with BayClone2. If not, see <http://www.gnu.org/licenses/>.
  ############################################################################ 
  

rm(list=ls(all=TRUE))
set.seed(79861)
source("../source/fn_BayClone2.R")
dyn.load("../source/BayClone2.so")

######################################
S <- 100 # THE NUMBER OF LOCI (I.E. NUMBER OF ROWS OF L AND Z)
T <- 25 #THE NUMBER OF TISSUE SAMPLES


########################################################
#SIMULATE DATA
##THIS GENERATES DATA USED FOR SIMULATION 1 IN BAYCLONE2: A BAYESIAN SEQUENCE AND COPY NUMBER CALLER FOR SUBCLONES USING NGS DATA (SUBMITTED FOR BIOINFORMATICS)
########################################################
#DATA
##PREPARE TWO MATRICS TO RECORD THE TOTAL NUMBER OS READS AND THE NUMBER OF READS AT LOCUS s (ROWS) AND SAMPLE t (COLUMS)
N <- array(NA, dim=c(S, T)) #TOTAL NUMBER OF READS
n <- array(NA, dim=c(S, T)) #NUMBER OF READS INDICATING MUTATIONS

#SIMULTATION TRUTH---SAVE ALL THE PARAMETERS USED TO GENERATE SIMULATED DATA
sim.tr <- NULL
sim.tr$C <- 5 #INCLUDING BACKGROUND

#SET L AND Z===== FOR EACH SUBCLONE (IN COLUMS), WE SPECIFIY L AND Z
sim.tr$L <- array(2, dim=c(S, sim.tr$C))
sim.tr$L[41:50,2] <- 0
sim.tr$L[51:60,2] <- 1

sim.tr$L[51:60,3] <- 0
sim.tr$L[61:70,3] <- 3

sim.tr$L[61:70,4] <- 1
sim.tr$L[71:80,4] <- 3

sim.tr$L[81:90,5] <- 0
sim.tr$L[91:100,5] <- 3

sim.tr$Z <- array(0, dim=c(S, sim.tr$C))
sim.tr$Z[,1] <- 2  ###BACKGROUND

sim.tr$Z[56:60,2] <- 1
sim.tr$Z[61:80, 2] <- 1
sim.tr$Z[81:100, 2] <- 2

sim.tr$Z[1:25, 3] <- 1
sim.tr$Z[26:50, 3] <- 2
sim.tr$Z[61:70, 3] <- c(0, 0, 0, 1, 1, 2, 2, 3, 3, 3)
sim.tr$Z[71:85, 3] <- 2

sim.tr$Z[1:30, 4] <- 2
sim.tr$Z[31:60, 4] <- 1
sim.tr$Z[61:70, 4] <- c(rep(0, 5), rep(1, 5))
sim.tr$Z[71:80, 4] <- c(0, 0, 0, 1, 1, 2, 2, 3, 3, 3)

sim.tr$Z[26:60, 5] <- 2
sim.tr$Z[61:80, 5] <- 1
sim.tr$Z[91:100, 5] <- c(0, 0, 0, 1, 1, 2, 2, 3, 3, 3)

##PHI---AVERAGE TOTAL NUMBER OF READS FOR EACH SAMPLE
sim.tr$phi <- rgamma(T, 600, 3)  #TOTAL NUMBER OF READS


###W--WEIHTS
sim.tr$w <- sim.tr$w_star <- array(NA, dim=c(T, sim.tr$C))
a.vec <- c(0.01, 0.4, 0.3, 0.15, 0.05)*30

for(i.t in 1:T) #FOR EACH SAMPLE, t,
{
    tmp <- sample(a.vec[-1], sim.tr$C-1)  ##RANDOM PERMUTATION OF A
    tmp <- c(a.vec[1], tmp)  #COLUMN 1 (BACKGROUND CELL TYPE GETS SMALL COUNTS)
    sim.tr$w_star[i.t, ] <- rgamma(sim.tr$C, tmp, 1)
    sim.tr$w[i.t,] <- sim.tr$w_star[i.t,]/sum(sim.tr$w_star[i.t,])
}

sim.tr$p_z0 <- c(0.05, rep(1, sim.tr$C-1))  ##P0: PROPORTION OF SNV IN THE BACKGROUND SUBCLONE


##COMPUTE THE SAMPLE COPY NUMBERS AND EXPECTED VARIANT ALLELE FRACTIONS.
sim.tr$M <- sim.tr$p <- array(NA, dim=c(S, T))
for(i.s in 1:S)
for(i.t in 1:T)
{
    sim.tr$M[i.s, i.t] <- sum(sim.tr$w[i.t,]*sim.tr$L[i.s,])
    sim.tr$p[i.s, i.t] <- sum(sim.tr$w[i.t,]*sim.tr$Z[i.s,]*sim.tr$p_z0)/sum(sim.tr$w[i.t,]*sim.tr$L[i.s,])
}

###FINALLY SIMULATE N AND n FOR EACH LOCUS AND EACH SAMPLE USING THE ASSUMED SAMPLING MODELS
N <- n <- array(0, dim=c(S, T))
for(i.t in 1:T)
{
    N[,i.t] <- rpois(S, sim.tr$phi[i.t]*sim.tr$M[,i.t]/2)  ### N_ST | M_ST, PH_T \SIM POISSON(PHI_T M_ST/2)
    n[,i.t] <- rbinom(S, N[,i.t], sim.tr$p[,i.t])  ####n_ST | N_ST, P_ST \SIM BINOMIAL(N_ST, P_ST)
}



###################################
#HYPER-PARAMETER  ----SPECIFYING HYPERPARAMETER VALUES
######################################
hyper <- NULL

#NUMBER OF SUBCLONES (GEOMETRIC DIST)
### C \SIM GEOMETRIC(R) WHERE E(C)=1/R
hyper$r <- 0.2

#PRIOR FOR cIBP
hyper$Q <- 3  # MAX NUMBER OF COPIES -- q = 0, 1, 2, 3

##BETA-DIRICHLET
###PI_C | C \SIM BETA-DIRICHLET (ALPHA/C, BETA, GAMMA)
hyper$alpha <- 3
hyper$beta <- 1
hyper$gam <- c(0.5, 0.5, 0.5)

#PRIOR FOR PHI--TOTAL NUMBER OF READS IN SAMPLE T
###PHI_T \SIM GAMMA(A, B)
hyper$a <- median(N)
hyper$b <- 1

#PRIOR FOR P_zO
###P0 \SIM BETA(a, b)
hyper$a_z0 <- 0.3
hyper$b_z0 <- 5

#PRIOR FOR W
##W_T | L \SIM DIRICHLET(D0, D, ..., D) WHERE W_T=(w_t0, w_t1, ..., w_tC)
hyper$d0 <- 0.5
hyper$d <- 1





################################################################################
#FIT THE MODEL
set.seed(11615)
#############################
#WE USE THE MCMC SIMULATION STRATEGY PROPOSED IN LEE AT EL (2014)
n.sam <- 10;  ##NUMBER OF SAMPLES THAT WILL BE USED FOR INFERENCE
burn.in <- 6  ##NUMBER OF SAMPLES FOR BURN-IN (USE THIS FOR A TRAINING DATA---FOR DETIALS, SEE THE REFERENCE
###########################
#SPLITTING TRAINING AND TESTING
##FOR DETIALS, SEE THE REFERENCE LEE AT EL (2014)
##B: FRACTION USED TO SPLIT DATA (B IS THE PROPORTION FOR A TRAINING DATASET)
##FROM OUR EXPERIENCE, 2.5% WORKS FINE. HOWEVER!!!!! YOU MAY NEED TO CHANGE THIS ACCORDING TO YOUR DATA!!!!!!!!!!!!!!!!!!!!!
B <- array(rbeta(S*T, 25, 975), dim=c(S, T))  ##FRACTION USED TO SPLIT DATA (B IS THE PROPORTION FOR A TRAINING DATASET)
#################!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
n_Tr <- n*B  ##n FOR TRAINING
n_Te <- n - n_Tr  ##n FOR TESTING

N_Tr <- N*B  ##N FOR TRAINING
N_Te <- N - N_Tr  ##N FOR TESTING

##PUT THE DATASETS IN A LONG SEQUENCE INSTEAD OF ARRAYS.
n_Tr.tmp <- array(n_Tr, dim=c(1, S*T))[1,]
N_Tr.tmp <- array(N_Tr, dim=c(1, S*T))[1,]
N_Tr_sum <- apply(N_Tr, 2, sum)

n_Te.tmp <- array(n_Te, dim=c(1, S*T))[1,]
N_Te.tmp <- array(N_Te, dim=c(1, S*T))[1,]
N_Te_sum <- apply(N_Te, 2, sum)

B_Tr.tmp <- array(B, dim=c(1, S*T))[1,]
B_Te.tmp <- array(1-B, dim=c(1, S*T))[1,]


n.tmp <- array(n, dim=c(1, S*T))[1,]
N.tmp <- array(N, dim=c(1, S*T))[1,]
N_sum <- apply(N, 2, sum)  ####COLUMN SUM OF N


##############################################
###WE CONSIDER C BETWEEN 1 AND 14
Min_C <- 2  #INCLUDING THE BACKGROUND SUBCLONE
Max_C <- 15  ##INCLUDING THE BACKGROUND SUBCLONE
n_C <- Max_C - Min_C + 1  ## NUMBER OF VALUES OF C CONSIDERED.

###########################################################################
###DO MCMC FOR TRAINING DATASET
###FOR EACH VALUE OF C, SAMPLE THE OTHER PARAMETERS INCLUDING L, Z, W, PHI, PI, P0 USING TRAINING DATA FOR BURN-IN ITERATIONS
sam.all.OR <- fn.Tr.CNV(hyper, S, T, N_Tr.tmp, n_Tr.tmp, N_Tr_sum, B_Tr.tmp, Min_C, Max_C, burn.in)


###NOW WE DO TRANS-DIMENSIONAL MCMC (THAT IS, SAMPLE C AS WELL AS THE OTHER PARAMETERS) USING THE TEST DATASET.
set.seed(115516131)
MCMC.sam <- fn.MCMC.CNV(sam.all.OR, hyper, S, T, N_Tr.tmp, n_Tr.tmp, N_Tr_sum, B_Tr.tmp, N_Te.tmp, n_Te.tmp, N_Te_sum, B_Te.tmp, N.tmp, n.tmp, N_sum, Min_C, Max_C, n.sam)

