/*#############################################################################
  
  # BayClone2 :  A Bayesian Sequence and Copy Number Caller for Subclones Using NGS Data
  # Copyright (C) <2014>  <Juhee Lee>
  # This file is part of BayClone2.

  #  BayClone2 is free software: you can redistribute it and/or modify
  #  it under the terms of the GNU General Public License as published by
  #  the Free Software Foundation, either version 3 of the License, or
  #  (at your option) any later version.

  #  BayClone2 is distributed in the hope that it will be useful,
  #  but WITHOUT ANY WARRANTY; without even the implied warranty of
  #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  #  GNU General Public License for more details.

  #  You should have received a copy of the GNU General Public License
  #  along with BayClone2. If not, see <http://www.gnu.org/licenses/>.
  ############################################################################ */

#include <stdio.h>
#include <R.h>
#include <Rmath.h>
#include <assert.h>
#include <math.h>

//#include <stddef.h>
//#include <veclib/clapack.h>
//#include "CT_rho_fn.h"


//R CMD SHLIB intratumor.cpp
extern "C"
{

    double myabs(double a)
	{
		return (a > 0.0) ? a : (-1.0*a);
	}

    void fn_sum_Z_1(int *SS, int *CC, int *ind_NN, int *ind_set, int *Z_1, int *Z_2, double *min_dist)
	{
		int S = *SS;
		int C = *CC;
        int ind_N = *ind_NN;
        
        int i1, i2, i_c, i_s, i_N;
        double diff_s, mmin_dist, tmp_min;
        
        double *D; D = new double[C*C];
        
        for (i1=1; i1 <= C; i1++) {
            for (i2=1; i2 <= C; i2++) {
                //DIFFERENCE IN Z_C
                diff_s = 0.0;
                for (i_s=0; i_s < S; i_s++) {
                    if (Z_1[i1*S + i_s] != Z_2[i2*S + i_s]) {
                        diff_s = diff_s + 1.0;
                    }
                }
                D[(i2-1)*C + (i1-1)] = diff_s;
                //printf("\n i1=%d, i2=%d, D=%f", i1, i2, D[(i2-1)*C + (i1-1)]);
            }
        }
        
        mmin_dist = INFINITY;
        
        for (i_N = 0; i_N < ind_N; i_N++) {
            tmp_min = 0.0;
            for (i_c=0; i_c < C; i_c++) {
                i1 = i_c;
                i2 = ind_set[ind_N*i_c + i_N];
                tmp_min = tmp_min + D[i1*C + i2];
                //printf("\n i_1=%d, i_2=%d, D=%f, tmp_min=%f", i1, i2, D[i1*C + i2], tmp_min);
            }
            
            if (tmp_min < mmin_dist) {
                mmin_dist = tmp_min;
            }
            //printf("\n tmp_min=%f, min_dist=%f", tmp_min, mmin_dist);
        }
        
        *min_dist = mmin_dist;
        delete[] D; D = NULL;
    }

/*	void fn_sum_Z_1(int *SS, int *TT, int *CC, int *ind_NN, int *ind_set, int *Z_1, int *Z_2, double *w_1, double *w_2, double *min_dist)
	{
		int T = *TT;
		int S = *SS;
		int C = *CC;
        int ind_N = *ind_NN;
        
        int i1, i2, i_c, i_t, i_s, i_N;
        double ave_w, diff_s, mmin_dist, tmp_min;
        
        double *D; D = new double[C*C];
        
        for (i1=1; i1 <= C; i1++) {
            for (i2=1; i2 <= C; i2++) {
                
                //AVERAGE W
                ave_w = 0.0;
                for (i_t = 0; i_t < T; i_t ++) {
                    ave_w = ave_w + myabs(w_1[i1*T + i_t] - w_2[i2*T + i_t]);
                }
                
                //DIFFERENCE IN Z_C
                diff_s = 0.0;
                for (i_s=0; i_s < S; i_s++) {
                    if (Z_1[i1*S + i_s] != Z_2[i2*S + i_s]) {
                        diff_s = diff_s + 1.0;
                    }
                }
                D[(i2-1)*C + (i1-1)] = diff_s*ave_w/(1.0*T);
                //printf("\n i1=%d, i2=%d, D=%f", i1, i2, D[(i2-1)*C + (i1-1)]);
            }
        }
        
        mmin_dist = INFINITY;
 
        for (i_N = 0; i_N < ind_N; i_N++) {
            tmp_min = 0.0;
            for (i_c=0; i_c < C; i_c++) {
                i1 = i_c;
                i2 = ind_set[ind_N*i_c + i_N];
                tmp_min = tmp_min + D[i1*C + i2];
                //printf("\n i_1=%d, i_2=%d, D=%f, tmp_min=%f", i1, i2, D[i1*C + i2], tmp_min);
            }
            
            if (tmp_min < mmin_dist) {
                mmin_dist = tmp_min;
            }
            //printf("\n tmp_min=%f, min_dist=%f", tmp_min, mmin_dist);
        }
        
        *min_dist = mmin_dist;
        delete[] D; D = NULL;
    }
 */
    
 }




